/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.isobus.model;

import java.io.DataOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.Writer;
import java.util.logging.Logger;

/**
 * Class which can be used to write binaries
 *
 * @author Gerben Feenstra
 */
public class BinaryWriter extends Writer {

    private static final Logger LOGGER = Logger.getLogger("nl.cloudfarming.client.isobus.model");
    private final DataOutputStream dataOutputStream;

    public BinaryWriter(OutputStream outputStream) {
        dataOutputStream = new DataOutputStream(outputStream);
    }

    /**
     * writes last byte of a long value to the wrapped data output stream.
     * Only least significant 8 bits are used, rest of the precision of the long
     * and the sign will be lost.
     *
     * ONLY SUPPORTS BIT8_UNSIGNED and BIT32_SIGNED AS OF YET!
     *
     * @param field
     * @param valueToWrite Value to write to this field
     */
    public void write(XmlField field, long valueToWrite) throws IOException {
        switch (field.getDataType()) {
            case BIT8_UNSIGNED:
                assert valueToWrite >= 0:"Value to write cannot be < 0";
                assert valueToWrite <= 255:"Value should be smaller than 255";
                dataOutputStream.write(new byte[]{(byte)valueToWrite});
                break;
            case BIT32_SIGNED:
                assert valueToWrite >= Integer.MIN_VALUE;
                assert valueToWrite <= Integer.MAX_VALUE;
                
                // little Endian
                dataOutputStream.write(new byte[]{
                    (byte) valueToWrite,
                    (byte)(valueToWrite >> 8),
                    (byte)(valueToWrite >> 16),
                    (byte)(valueToWrite >> 24),
                });
                break;
            default:
                throw new IllegalArgumentException("data type " + field.getDataType() + " is not supported yet");
        }

    }

    @Override
    public void write(char[] cbuf, int off, int len) throws IOException {
        throw new UnsupportedOperationException("writing of character arrays not supported");
    }

    @Override
    public void flush() throws IOException {
        dataOutputStream.flush();
    }

    @Override
    public void close() throws IOException {
        dataOutputStream.close();
    }
}
