/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.isobus.model;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import javax.swing.AbstractAction;
import javax.swing.JFileChooser;
import nl.cloudfarming.client.task.ExportableTask;
import nl.cloudfarming.client.task.ExportableTaskCapability;
import nl.cloudfarming.client.util.Callback;
import org.openide.awt.ActionID;
import org.openide.awt.ActionRegistration;
import org.openide.filesystems.FileChooserBuilder;
import org.openide.filesystems.FileObject;
import org.openide.filesystems.FileUtil;
import org.openide.util.Exceptions;
import org.openide.util.Lookup;
import org.openide.util.NbBundle;

/**
 *
 * @author Johan Reitzema
 * @author Maciek Dulkiewicz
 */
@ActionID(category = "ExportableTask", id = "nl.cloudfarming.client.isobus.type.ExportAction")
@ActionRegistration(displayName = "#isobus.action.export")
@NbBundle.Messages({
    "isobus.action.export=Export Isobus",
    "isobus.action.export.chooselocation=Choose export location",
    "isobus.action.progress_handler_msg=Exporting Isobus task data",
    "isobus.action.confirm_overwrite_taskdata=Overwrite current taskdata folder contents?"    
})
public class ExportAction extends AbstractAction {
    public static final DDI DEFAULT_DDI = DDI.DDI_141; // fall back to on/off value which was previously hardcoded for all exports.
    public static final double DEFAULT_MULTIPLIER = 1.0;
    
    private final List<ExportableTaskCapability> exportCapabilities;
    private final DDI ddi;
    private final double multiplier;
    private Callback<Object> callback;

    public ExportAction(List<ExportableTaskCapability> exportCapabilities) {
        this(exportCapabilities, DEFAULT_DDI, DEFAULT_MULTIPLIER, null);
    }
    
    private ExportAction(List<ExportableTaskCapability> exportCapabilities, DDI ddi, double multiplier, Callback<Object> callback) {
        super(NbBundle.getMessage(ExportAction.class, "isobus.action.export"));
        this.exportCapabilities = exportCapabilities;
        this.ddi = ddi;
        this.multiplier = multiplier;
        this.callback = callback;
    }

    @Override
    public void actionPerformed(ActionEvent e) {
        FileChooserBuilder builder = new FileChooserBuilder(ExportAction.class);
        builder.setTitle(Bundle.isobus_action_export_chooselocation());

        final JFileChooser chooser = builder.createFileChooser();
        chooser.setMultiSelectionEnabled(false);
        chooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
        chooser.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent e) {
                if (JFileChooser.APPROVE_SELECTION.equals(e.getActionCommand())) {
                    if (chooser.getSelectedFile() != null) {
                        FileObject selectedLocation = FileUtil.toFileObject(chooser.getSelectedFile());
                        FileObject targetLocation = getTargetLocation(selectedLocation);
                        if (clearTargetContents(targetLocation)) {
                            List<ExportableTask> tasksToExport = new ArrayList<>();
                            for (ExportableTaskCapability capability : exportCapabilities) {
                                tasksToExport.add(capability.getTask());
                            }

                            GridTypeEnum exportType = GridTypeEnum.valueOf(IsobusPreference.EXPORT_TYPE.getValue());
                            IsobusManager isobusManager = Lookup.getDefault().lookup(IsobusManager.class);
                            isobusManager.exportTaskdata(targetLocation, exportType, ddi, multiplier, callback, tasksToExport.toArray(new ExportableTask[tasksToExport.size()]));
                        }
                    }
                }
            }
        });

        chooser.showOpenDialog(null);
    }
    
    private static final String EXPORT_FOLDER_NAME = "TASKDATA";
    
    private FileObject getTargetLocation(FileObject selectedLocation) {
        // TODO: 
        // - confirm overwriting existing taskdata; 
        // - when confirmed, clear existing files first (previous export might have more grid binaries than the current one)
        if (selectedLocation.getName().equals(EXPORT_FOLDER_NAME)) {
            return selectedLocation;
        }
        
        FileObject subFolder = selectedLocation.getFileObject(EXPORT_FOLDER_NAME);
        if (subFolder == null) {
            try {
                subFolder = selectedLocation.createFolder(EXPORT_FOLDER_NAME);
            } catch (IOException ex) {
                throw new RuntimeException("Could not create export subfolder", ex);
            }
        }
        return subFolder;
    }
    
    private boolean clearTargetContents(FileObject targetLocation) {
        if (targetLocation.getChildren().length > 0) {
            // TODO: confirm overwrite?
            for (FileObject fob : targetLocation.getChildren()) {
                try {
                    fob.delete();
                } catch (IOException ex) {
                    Exceptions.printStackTrace(ex);
                    return false;
                }
            }
        }
        return true;
    }
    
    //FIXME: isobus DDI/UOM info should be included in the task, but will cause a circular dependency with the current module structure.
    // also: workaround for Lookup injection of ctor parameter, which can handle only one public constructor
    public static ExportAction create(List<ExportableTaskCapability> exportCapabilities, DDI ddi, double multiplier, Callback<Object> callback) {
        return new ExportAction(exportCapabilities, ddi, multiplier, callback);
    }
}
