/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.isobus.model;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.bind.Unmarshaller;
import nl.cloudfarming.client.logging.AppLogFactory;
import nl.cloudfarming.client.logging.AppLogger;
import org.openide.filesystems.FileObject;
import org.openide.util.Exceptions;
import org.openide.util.NbBundle;

/**
 * UnMarshaller to unMarshall iso 11783-10 messages
 *
 * @author Merijn Zengers & Gerben Feenstra
 */
@NbBundle.Messages({
    "isobus.error.isobus_unmarshaller.exception=The isobus file {0} could not be read!",
    "isobus.error.isobus_unmarshaller.timelog_not_found=File {0} cannot be found!",
    "isobus.error.isobus_unmarshaller.xml_cannot_be_null=the XMLFile provided cannot be null"
})
public class IsobusMarshaller {

    private static final AppLogger LOG = AppLogFactory.getLogger(IsobusMarshaller.class);
    private static final Logger LOGGER = Logger.getLogger("nl.cloudfarming.client.isobus.model");
    private final String PACKAGE_NAME = "nl.cloudfarming.client.isobus.model";
    private static final String DEFAULT_TASKDATA_FILENAME = "TASKDATA";
    private static final String DEFAULT_TASKDATA_FILENAME_EXT = "XML";

    /**
     * Marshalls the provided IsobusTaskData to the supplied FileObject
     *
     * @param targetFolder the fileObject to write to
     * @param isobusTaskData the IsobusTaskData to marshall
     */
    public void marshall(FileObject targetFolder, IsobusTaskData isobusTaskData) {
        JAXBContext jc;

        try {
            jc = JAXBContext.newInstance(PACKAGE_NAME);
            Marshaller marshaller = jc.createMarshaller();
            // remove old file first if it exists:
            FileObject targetFile = targetFolder.getFileObject(DEFAULT_TASKDATA_FILENAME, DEFAULT_TASKDATA_FILENAME_EXT);
            if (targetFile != null) {
                targetFile.delete();
            }
            targetFile = targetFolder.createData(DEFAULT_TASKDATA_FILENAME, DEFAULT_TASKDATA_FILENAME_EXT);
            try (OutputStream outputStream = targetFile.getOutputStream()) {
                marshaller.marshal(isobusTaskData, outputStream);
            }
        } catch (IOException | JAXBException ex) {
            LOG.error("Failed to marshall Isobus taskdata to xml");
            LOGGER.log(Level.SEVERE, "Failed to marshall Isobus taskdata to xml {0}", ex.getMessage());
        }
    }

    /**
     * UnMarshal File containing XML to an IsobusTaskData object
     *
     * @param xmlFile The File object which contains the XML
     * @return An IsobusTaskData object
     */
    public IsobusTaskData unMarshall(FileObject xmlFile) throws IllegalArgumentException {

        // XML should not be null
        if (xmlFile == null) {
            throw new IllegalArgumentException(NbBundle.getMessage(IsobusMarshaller.class, "isobus.error.isobus_unmarshaller.xml_cannot_be_null"));
        }

        // Unmarshall XML
        IsobusTaskData isobusTaskData = null;
        try {
            isobusTaskData = unMarshallIsobusTaskData(xmlFile.getInputStream());
        } catch (FileNotFoundException ex) {
            Exceptions.printStackTrace(ex);
        }
        return isobusTaskData;
    }

    /**
     * Unmarshall the specified InputStream to an IsobusTaskData object. Do not
     * use, PROTECTED for testing only!
     *
     * @param taskdata Taskdata as inputstream
     * @return IsobusTaskData Unmarshalled isobus taskdata object
     */
    protected IsobusTaskData unMarshallIsobusTaskData(InputStream taskdata) {
        JAXBContext jc;

        try {
            jc = JAXBContext.newInstance(PACKAGE_NAME);
            Unmarshaller unmarshaller = jc.createUnmarshaller();
            IsobusTaskData isobusTaskData = (IsobusTaskData) unmarshaller.unmarshal(taskdata);
            return isobusTaskData;
        } catch (JAXBException ex) {
            LOG.error("Failed to convert xml", ex);
        }
        return null;
    }
}
