/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package nl.cloudfarming.client.isobus.model.util;

import com.vividsolutions.jts.geom.Coordinate;
import com.vividsolutions.jts.geom.Geometry;
import com.vividsolutions.jts.geom.GeometryCollection;
import java.util.Collection;
import nl.cloudfarming.client.field.model.TreatmentZone;

/**
 * Util class for geometries
 * Can provide the envelope for a collection of treatmentzones
 * Can provide the cellWidth and height for a certain envelope an grid columns rows
 * 
 * @author Merijn Zengers
 */
public class GeometryUtil {
    
    /**
     * Calculate grid cell width size
     *
     * @param envelope
     * @param nrOfGridColumns
     * @return
     */
    public static double getCellWidth(Geometry envelope, int nrOfGridColumns) {
        double envelopeMinX = getMinX(envelope);
        double envelopeMaxX = getMaxX(envelope);
        double width = envelopeMaxX - envelopeMinX;
        return width / nrOfGridColumns;
    }

    /**
     * Calculate grid cell height size
     *
     * @param envelope the envelope to calculate for
     * @param nrOfGridRows the number of rows in the grid
     * @return 
     */
    public static double getCellHeight(Geometry envelope, int nrOfGridRows) {
        double envelopeMinY = getMinY(envelope);
        double envelopeMaxY = getMaxY(envelope);
        double height = envelopeMaxY - envelopeMinY;
        return height / nrOfGridRows;
    }

    /**
     * Calculates the enveloper of the specified treatmentZones as a whole
     *
     * @param treatmentZones TreatmentZone to calculate envelope from
     * @return The envelope as a Geometry
     */
    public static Geometry getEnvelope(Collection<TreatmentZone> treatmentZones) {
        // Determine total envelope
        Geometry[] geometries = new Geometry[treatmentZones.size()];
        int counter = 0;
        for (TreatmentZone treatmentZone : treatmentZones) {
            geometries[counter] = treatmentZone.getGeometry();
            counter++;
        }
        GeometryCollection collection = new GeometryCollection(geometries, geometries[0].getFactory());
        return collection.getEnvelope();
    }
    
    /**
     * Get the largest X coordinate of the specified geometry
     * @param geometry Geometry to calculate largest X coordinate from
     * @return largest X in the geometry
     */
    public static double getMaxX(Geometry geometry){
        double maxX = geometry.getCoordinates()[0].x;
        for(Coordinate coordinate : geometry.getCoordinates()){
            if(coordinate.x > maxX){
                maxX = coordinate.x;
            }
        }
        return maxX;
    }
    
    /**
     * Get the smallest X coordinate of the specified geometry
     * @param geometry Geometry to calculate smallest X value from
     * @return smallest X in the geometry
     */
    public static double getMinX(Geometry geometry){
        double minX = geometry.getCoordinates()[0].x;
        for(Coordinate coordinate : geometry.getCoordinates()){
            if(coordinate.x < minX){
                minX = coordinate.x;
            }
        }
        return minX;
    }
    
    /**
     * Get the largest Y coordinate of the specified geometry
     * @param geometry Geometry to calculate largest Y coordinate from
     * @return largest Y in the geometry
     */
    public static double getMaxY(Geometry geometry){
        double maxY = geometry.getCoordinates()[0].y;
        for(Coordinate coordinate : geometry.getCoordinates()){
            if(coordinate.y > maxY){
                maxY = coordinate.y;
            }
            
        }
        return maxY;
    }
    
    /**
     * Get the smallest Y coordinate of the specified geometry
     * @param geometry Geometry to calculate smallest Y value from
     * @return smallest Y in the geometry
     */
    public static double getMinY(Geometry geometry){
        double minY = geometry.getCoordinates()[0].y;
        for(Coordinate coordinate : geometry.getCoordinates()){
            if(coordinate.y < minY){
                minY = coordinate.y;
            }
        }
        return minY;
    }
}
