/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.isobus.model.util;

import java.math.BigDecimal;
import java.util.logging.Level;
import java.util.logging.Logger;
import nl.cloudfarming.client.field.model.TreatmentZone;
import nl.cloudfarming.client.isobus.model.*;
import static nl.cloudfarming.client.isobus.model.util.DatatypeConverters.limitDecimalPlaces;
import nl.cloudfarming.client.task.grid.TreatmentZoneGrid;

/**
 * Can create a grid
 *
 * @author Merijn Zengers
 */
public class IsobusObjectFactoryUtil {

    private static final Logger LOGGER = Logger.getLogger("nl.cloudfarming.client.isobus.model.util");
    
    public static final int GRID_CELL_SIZE_DECIMALS = 8;
    public static final int GRID_POSITION_DECIMALS = 8;

    /**
     * Will create an IsobusGrid object using the passed
     * {@link TreatmentZoneGrid}.
     *
     * @param TreatmentZoneGrid the TreatmentZoneGrid used as the base for this
     * IsobusGrid
     * @param gridType the type of grid that will be created
     * @param gridFileName the filename used for the grid
     * @param treatMentZoneCode only required for the {@link GridTypeEnum#TYPE_2}
     * the code of the only {@link IsobusTreatmentZone} in the task
     * @return the created IsobusGrid
     */
    public static IsobusGrid createGrid(TreatmentZoneGrid treatmentZoneGrid, GridTypeEnum gridType, String gridFileName, Short treatMentZoneCode) {
        if (GridTypeEnum.TYPE_2.equals(gridType) && treatMentZoneCode == null) {
            throw new IllegalArgumentException("Grid type 2 needs a treatment zone code");
        }
        
        IsobusGrid grid = new IsobusGrid();
        grid.setMaximumColumn(treatmentZoneGrid.getNumberOfColumns());
        grid.setMaximumRow(treatmentZoneGrid.getNumberOfRows());
        grid.setCellEastSize(treatmentZoneGrid.getCellWidth());
        grid.setCellNorthSize(treatmentZoneGrid.getCellHeight());
        grid.setMinimumEastPosition(limitDecimalPlaces(new BigDecimal(treatmentZoneGrid.getLongitude()), GRID_POSITION_DECIMALS));
        grid.setMinimumNorthPosition(limitDecimalPlaces(new BigDecimal(treatmentZoneGrid.getLatitude()), GRID_POSITION_DECIMALS));
        grid.setType(gridType);
        grid.setFilename(gridFileName);
        if (GridTypeEnum.TYPE_2.equals(gridType)) {
            grid.setTreatmentZoneCode(treatMentZoneCode);
        }
        return grid;
    }
    
    /**
     * Create an IsobusTreatmentZone using an Agrosense TreatmentZone and a
     * TreatmentzoneCode for the task in the context of this treatmentzone. Will
     * add a ProcessDataVariable to the created IsobusTreatmentZone Will set the
     * processDataDDI to hex 8D as provided in ISO_11783-11
     *
     * @param presentation may be null
     * @param treatmentZone Agrosense Treatmentzone
     * @param treatmentZoneCode Code
     * @return Isobus Treatmentzone
     */
    public static IsobusTreatmentZone createIsobusTreatmentZone(ValuePresentation presentation, TreatmentZone treatmentZone, short treatmentZoneCode, DDI ddi, double multiplier) {
        if (treatmentZone.getValue() != null) {
            IsobusTreatmentZone isobusTreatmentZone = new IsobusTreatmentZone();
            isobusTreatmentZone.setTreatmentZoneDesignator(treatmentZone.getName());
            isobusTreatmentZone.setTreatmentZoneCode(treatmentZoneCode);
            isobusTreatmentZone.setTreatmentZoneColor(new Double(treatmentZone.getValue()).shortValue());

            // ProcessDataVariable for this treatmentzone
            ProcessDataVariable processDataVariable = createProcessDataVariable(treatmentZone.getValue(), presentation, ddi, multiplier);
            isobusTreatmentZone.getPDV().add(processDataVariable);
            return isobusTreatmentZone;
        } else {
            LOGGER.log(Level.SEVERE, "Treatment zone has no value set cannot create isobus treatment zone from it {0}", treatmentZone.getName());
        }
        return null;
    }

    /**
     * Create a {@link ProcessDataVariable} with the value and {@link ValuePresentation}
     * passed
     *
     * @param value the value to set
     * @param presentation the valuePresentation to set
     * @param ddi isobus data dictionary id
     * @return the created ProcesDataVariable
     */
    public static ProcessDataVariable createProcessDataVariable(double value, ValuePresentation presentation, DDI ddi, double multiplier) {
        ProcessDataVariable processDataVariable = new ProcessDataVariable();

        processDataVariable.setProcessDataDDI(ddi.getDDEntityId()); // A

        processDataVariable.setProcessDataValue(ddi.toExportValue(value * multiplier)); // B

        processDataVariable.setValuePresentationIdRef(presentation);
        return processDataVariable;
    }
}
